@@ -1,7 +1,5 @@
 // Take a look at the license at the top of the repository in the LICENSE file.
 
-use thiserror::Error;
-
 use glib::prelude::*;
 use glib::translate::*;
 
@@ -26,10 +24,13 @@ trait EncodingProfileBuilderCommon {
     fn set_preset(&self, preset: Option<&str>);
 
     fn set_preset_name(&self, preset_name: Option<&str>);
+
+    #[cfg(feature = "v1_18")]
+    fn set_single_segment(&self, single_segment: bool);
 }
 
 impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
-    #[doc(alias = "gst_encoding_profile_set_allow_dynamic_output")]
+    // checker-ignore-item
     fn set_allow_dynamic_output(&self, allow_dynamic_output: bool) {
         unsafe {
             ffi::gst_encoding_profile_set_allow_dynamic_output(
@@ -39,7 +40,7 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_description")]
+    // checker-ignore-item
     fn set_description(&self, description: Option<&str>) {
         let description = description.to_glib_none();
         unsafe {
@@ -50,7 +51,7 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_enabled")]
+    // checker-ignore-item
     fn set_enabled(&self, enabled: bool) {
         unsafe {
             ffi::gst_encoding_profile_set_enabled(
@@ -60,7 +61,7 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_format")]
+    // checker-ignore-item
     fn set_format(&self, format: &gst::Caps) {
         unsafe {
             ffi::gst_encoding_profile_set_format(
@@ -70,7 +71,7 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_name")]
+    // checker-ignore-item
     fn set_name(&self, name: Option<&str>) {
         let name = name.to_glib_none();
         unsafe {
@@ -78,14 +79,14 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_presence")]
+    // checker-ignore-item
     fn set_presence(&self, presence: u32) {
         unsafe {
             ffi::gst_encoding_profile_set_presence(self.as_ref().to_glib_none().0, presence);
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_preset")]
+    // checker-ignore-item
     fn set_preset(&self, preset: Option<&str>) {
         let preset = preset.to_glib_none();
         unsafe {
@@ -93,7 +94,7 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
         }
     }
 
-    #[doc(alias = "gst_encoding_profile_set_preset_name")]
+    // checker-ignore-item
     fn set_preset_name(&self, preset_name: Option<&str>) {
         let preset_name = preset_name.to_glib_none();
         unsafe {
@@ -103,6 +104,17 @@ impl<O: IsA<EncodingProfile>> EncodingProfileBuilderCommon for O {
             );
         }
     }
+
+    // checker-ignore-item
+    #[cfg(feature = "v1_18")]
+    fn set_single_segment(&self, single_segment: bool) {
+        unsafe {
+            ffi::gst_encoding_profile_set_single_segment(
+                self.as_ref().to_glib_none().0,
+                single_segment.into_glib(),
+            );
+        }
+    }
 }
 
 // Split the trait as only the getter is public
@@ -112,13 +124,14 @@ trait EncodingProfileHasRestrictionSetter {
 
 pub trait EncodingProfileHasRestrictionGetter {
     #[doc(alias = "get_restriction")]
+    #[doc(alias = "gst_encoding_profile_get_restriction")]
     fn restriction(&self) -> Option<gst::Caps>;
 }
 
 macro_rules! declare_encoding_profile_has_restriction(
     ($name:ident) => {
         impl EncodingProfileHasRestrictionSetter for $name {
-            #[doc(alias = "gst_encoding_profile_set_restriction")]
+            // checker-ignore-item
             fn set_restriction(&self, restriction: Option<&gst::Caps>) {
                 let profile: &EncodingProfile = glib::object::Cast::upcast_ref(self);
 
@@ -137,7 +150,7 @@ macro_rules! declare_encoding_profile_has_restriction(
         }
 
         impl EncodingProfileHasRestrictionGetter for $name {
-            #[doc(alias = "gst_encoding_profile_get_restriction")]
+            // checker-ignore-item
             fn restriction(&self) -> Option<gst::Caps> {
                 let profile: &EncodingProfile = glib::object::Cast::upcast_ref(self);
 
@@ -152,14 +165,14 @@ macro_rules! declare_encoding_profile_has_restriction(
 );
 
 impl EncodingAudioProfile {
-    #[doc(alias = "gst_encoding_audio_profile_new")]
+    // checker-ignore-item
     fn new(
         format: &gst::Caps,
         preset: Option<&str>,
         restriction: Option<&gst::Caps>,
         presence: u32,
     ) -> EncodingAudioProfile {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let preset = preset.to_glib_none();
         let restriction = restriction.to_glib_none();
         unsafe {
@@ -171,19 +184,25 @@ impl EncodingAudioProfile {
             ))
         }
     }
+
+    #[doc(alias = "gst_encoding_audio_profile_new")]
+    pub fn builder(format: &gst::Caps) -> EncodingAudioProfileBuilder {
+        assert_initialized_main_thread!();
+        EncodingAudioProfileBuilder::new(format)
+    }
 }
 
 declare_encoding_profile_has_restriction!(EncodingAudioProfile);
 
 impl EncodingVideoProfile {
-    #[doc(alias = "gst_encoding_video_profile_new")]
+    // checker-ignore-item
     fn new(
         format: &gst::Caps,
         preset: Option<&str>,
         restriction: Option<&gst::Caps>,
         presence: u32,
     ) -> EncodingVideoProfile {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let preset = preset.to_glib_none();
         let restriction = restriction.to_glib_none();
         unsafe {
@@ -196,14 +215,20 @@ impl EncodingVideoProfile {
         }
     }
 
-    #[doc(alias = "gst_encoding_video_profile_set_pass")]
+    #[doc(alias = "gst_encoding_video_profile_new")]
+    pub fn builder(format: &gst::Caps) -> EncodingVideoProfileBuilder {
+        assert_initialized_main_thread!();
+        EncodingVideoProfileBuilder::new(format)
+    }
+
+    // checker-ignore-item
     fn set_pass(&self, pass: u32) {
         unsafe {
             ffi::gst_encoding_video_profile_set_pass(self.to_glib_none().0, pass);
         }
     }
 
-    #[doc(alias = "gst_encoding_video_profile_set_variableframerate")]
+    // checker-ignore-item
     fn set_variableframerate(&self, variableframerate: bool) {
         unsafe {
             ffi::gst_encoding_video_profile_set_variableframerate(
@@ -217,14 +242,14 @@ impl EncodingVideoProfile {
 declare_encoding_profile_has_restriction!(EncodingVideoProfile);
 
 impl EncodingContainerProfile {
-    #[doc(alias = "gst_encoding_container_profile_new")]
+    // checker-ignore-item
     fn new(
         name: Option<&str>,
         description: Option<&str>,
         format: &gst::Caps,
         preset: Option<&str>,
     ) -> EncodingContainerProfile {
-        assert_initialized_main_thread!();
+        skip_assert_initialized!();
         let name = name.to_glib_none();
         let description = description.to_glib_none();
         let preset = preset.to_glib_none();
@@ -238,73 +263,87 @@ impl EncodingContainerProfile {
         }
     }
 
-    #[doc(alias = "gst_encoding_container_profile_add_profile")]
-    fn add_profile<P: IsA<EncodingProfile>>(
-        &self,
-        profile: &P,
-    ) -> Result<(), glib::error::BoolError> {
+    #[doc(alias = "gst_encoding_container_profile_new")]
+    pub fn builder(format: &gst::Caps) -> EncodingContainerProfileBuilder {
+        assert_initialized_main_thread!();
+        EncodingContainerProfileBuilder::new(format)
+    }
+
+    // checker-ignore-item
+    fn add_profile<P: IsA<EncodingProfile>>(&self, profile: &P) {
         unsafe {
-            glib::result_from_gboolean!(
-                ffi::gst_encoding_container_profile_add_profile(
-                    self.to_glib_none().0,
-                    profile.as_ref().to_glib_full(),
-                ),
-                "Failed to add profile",
-            )
+            let res = ffi::gst_encoding_container_profile_add_profile(
+                self.to_glib_none().0,
+                profile.as_ref().to_glib_full(),
+            );
+            // Can't possibly fail unless we pass random pointers
+            assert_ne!(res, glib::ffi::GFALSE);
         }
     }
 }
 
-#[derive(Debug, Clone, Error)]
-#[error("failed to build encoding profile")]
-pub struct EncodingProfileBuilderError(());
-
 #[derive(Debug)]
 struct EncodingProfileBuilderCommonData<'a> {
+    format: &'a gst::Caps,
     name: Option<&'a str>,
     description: Option<&'a str>,
-    format: Option<&'a gst::Caps>,
     preset: Option<&'a str>,
     preset_name: Option<&'a str>,
     presence: u32,
     allow_dynamic_output: bool,
     enabled: bool,
+    #[cfg(feature = "v1_18")]
+    single_segment: bool,
 }
 
 impl<'a> EncodingProfileBuilderCommonData<'a> {
-    fn new() -> EncodingProfileBuilderCommonData<'a> {
+    fn new(format: &'a gst::Caps) -> EncodingProfileBuilderCommonData<'a> {
+        skip_assert_initialized!();
         EncodingProfileBuilderCommonData {
             name: None,
             description: None,
-            format: None,
+            format,
             preset: None,
             preset_name: None,
             presence: 0,
             allow_dynamic_output: true,
             enabled: true,
+            #[cfg(feature = "v1_18")]
+            single_segment: false,
         }
     }
 }
 
 pub trait EncodingProfileBuilder<'a>: Sized {
+    #[doc(alias = "gst_encoding_profile_set_name")]
+    #[must_use]
     fn name(self, name: &'a str) -> Self;
+    #[doc(alias = "gst_encoding_profile_set_description")]
+    #[must_use]
     fn description(self, description: &'a str) -> Self;
-    fn format(self, format: &'a gst::Caps) -> Self;
+    #[doc(alias = "gst_encoding_profile_set_preset")]
+    #[must_use]
     fn preset(self, preset: &'a str) -> Self;
+    #[doc(alias = "gst_encoding_profile_set_preset_name")]
+    #[must_use]
     fn preset_name(self, preset_name: &'a str) -> Self;
+    #[doc(alias = "gst_encoding_profile_set_presence")]
+    #[must_use]
     fn presence(self, presence: u32) -> Self;
+    #[doc(alias = "gst_encoding_profile_set_allow_dynamic_output")]
+    #[must_use]
     fn allow_dynamic_output(self, allow: bool) -> Self;
+    #[doc(alias = "gst_encoding_profile_set_enabled")]
+    #[must_use]
     fn enabled(self, enabled: bool) -> Self;
+    #[cfg(feature = "v1_18")]
+    #[doc(alias = "gst_encoding_profile_set_single_segment")]
+    #[must_use]
+    fn single_segment(self, single_segment: bool) -> Self;
 }
 
 macro_rules! declare_encoding_profile_builder_common(
     ($name:ident) => {
-        impl<'a> Default for $name<'a> {
-            fn default() -> Self {
-                Self::new()
-            }
-        }
-
         impl<'a> EncodingProfileBuilder<'a> for $name<'a> {
             fn name(mut self, name: &'a str) -> $name<'a> {
                 self.base.name = Some(name);
@@ -316,11 +355,6 @@ macro_rules! declare_encoding_profile_builder_common(
                 self
             }
 
-            fn format(mut self, format: &'a gst::Caps) -> $name<'a> {
-                self.base.format = Some(format);
-                self
-            }
-
             fn preset(mut self, preset: &'a str) -> $name<'a> {
                 self.base.preset = Some(preset);
                 self
@@ -345,6 +379,12 @@ macro_rules! declare_encoding_profile_builder_common(
                 self.base.enabled = enabled;
                 self
             }
+
+            #[cfg(feature = "v1_18")]
+            fn single_segment(mut self, single_segment: bool) -> $name<'a> {
+                self.base.single_segment = single_segment;
+                self
+            }
         }
     }
 );
@@ -361,9 +401,14 @@ fn set_common_fields<T: EncodingProfileBuilderCommon>(
     profile.set_allow_dynamic_output(base_data.allow_dynamic_output);
     profile.set_enabled(base_data.enabled);
     profile.set_presence(base_data.presence);
+    #[cfg(feature = "v1_18")]
+    {
+        profile.set_single_segment(base_data.single_segment);
+    }
 }
 
 #[derive(Debug)]
+#[must_use = "The builder must be built to be used"]
 pub struct EncodingAudioProfileBuilder<'a> {
     base: EncodingProfileBuilderCommonData<'a>,
     restriction: Option<&'a gst::Caps>,
@@ -372,36 +417,37 @@ pub struct EncodingAudioProfileBuilder<'a> {
 declare_encoding_profile_builder_common!(EncodingAudioProfileBuilder);
 
 impl<'a> EncodingAudioProfileBuilder<'a> {
-    pub fn new() -> Self {
+    fn new(format: &'a gst::Caps) -> Self {
+        skip_assert_initialized!();
         EncodingAudioProfileBuilder {
-            base: EncodingProfileBuilderCommonData::new(),
+            base: EncodingProfileBuilderCommonData::new(format),
             restriction: None,
         }
     }
 
+    #[doc(alias = "gst_encoding_profile_set_restriction")]
     pub fn restriction(mut self, restriction: &'a gst::Caps) -> Self {
         self.restriction = Some(restriction);
         self
     }
 
-    pub fn build(self) -> Result<EncodingAudioProfile, EncodingProfileBuilderError> {
-        if self.base.format.is_none() {
-            return Err(EncodingProfileBuilderError(()));
-        }
-
+    #[must_use = "Building the profile without using it has no effect"]
+    pub fn build(self) -> EncodingAudioProfile {
         let profile = EncodingAudioProfile::new(
-            self.base.format.unwrap(),
+            self.base.format,
             self.base.preset,
             self.restriction,
             self.base.presence,
         );
 
         set_common_fields(&profile, &self.base);
-        Ok(profile)
+
+        profile
     }
 }
 
 #[derive(Debug)]
+#[must_use = "The builder must be built to be used"]
 pub struct EncodingVideoProfileBuilder<'a> {
     base: EncodingProfileBuilderCommonData<'a>,
     restriction: Option<&'a gst::Caps>,
@@ -412,37 +458,38 @@ pub struct EncodingVideoProfileBuilder<'a> {
 declare_encoding_profile_builder_common!(EncodingVideoProfileBuilder);
 
 impl<'a> EncodingVideoProfileBuilder<'a> {
-    pub fn new() -> Self {
+    fn new(format: &'a gst::Caps) -> Self {
+        skip_assert_initialized!();
         EncodingVideoProfileBuilder {
-            base: EncodingProfileBuilderCommonData::new(),
+            base: EncodingProfileBuilderCommonData::new(format),
             restriction: None,
             pass: 0,
             variable_framerate: false,
         }
     }
 
+    #[doc(alias = "gst_encoding_video_profile_set_pass")]
     pub fn pass(mut self, pass: u32) -> Self {
         self.pass = pass;
         self
     }
 
+    #[doc(alias = "gst_encoding_video_profile_set_variableframerate")]
     pub fn variable_framerate(mut self, variable_framerate: bool) -> Self {
         self.variable_framerate = variable_framerate;
         self
     }
 
+    #[doc(alias = "gst_encoding_profile_set_restriction")]
     pub fn restriction(mut self, restriction: &'a gst::Caps) -> Self {
         self.restriction = Some(restriction);
         self
     }
 
-    pub fn build(self) -> Result<EncodingVideoProfile, EncodingProfileBuilderError> {
-        if self.base.format.is_none() {
-            return Err(EncodingProfileBuilderError(()));
-        }
-
+    #[must_use = "Building the profile without using it has no effect"]
+    pub fn build(self) -> EncodingVideoProfile {
         let video_profile = EncodingVideoProfile::new(
-            self.base.format.unwrap(),
+            self.base.format,
             self.base.preset,
             self.restriction,
             self.base.presence,
@@ -452,11 +499,13 @@ impl<'a> EncodingVideoProfileBuilder<'a> {
         video_profile.set_variableframerate(self.variable_framerate);
 
         set_common_fields(&video_profile, &self.base);
-        Ok(video_profile)
+
+        video_profile
     }
 }
 
 #[derive(Debug)]
+#[must_use = "The builder must be built to be used"]
 pub struct EncodingContainerProfileBuilder<'a> {
     base: EncodingProfileBuilderCommonData<'a>,
     profiles: Vec<EncodingProfile>,
@@ -465,35 +514,33 @@ pub struct EncodingContainerProfileBuilder<'a> {
 declare_encoding_profile_builder_common!(EncodingContainerProfileBuilder);
 
 impl<'a> EncodingContainerProfileBuilder<'a> {
-    pub fn new() -> Self {
+    fn new(format: &'a gst::Caps) -> Self {
+        skip_assert_initialized!();
         EncodingContainerProfileBuilder {
-            base: EncodingProfileBuilderCommonData::new(),
+            base: EncodingProfileBuilderCommonData::new(format),
             profiles: Vec::new(),
         }
     }
 
-    pub fn build(self) -> Result<EncodingContainerProfile, EncodingProfileBuilderError> {
-        if self.base.format.is_none() {
-            return Err(EncodingProfileBuilderError(()));
-        }
-
+    #[must_use = "Building the profile without using it has no effect"]
+    pub fn build(self) -> EncodingContainerProfile {
         let container_profile = EncodingContainerProfile::new(
             self.base.name,
             self.base.description,
-            self.base.format.unwrap(),
+            self.base.format,
             self.base.preset,
         );
 
         for profile in self.profiles {
-            container_profile
-                .add_profile(&profile)
-                .map_err(|_error| EncodingProfileBuilderError(()))?;
+            container_profile.add_profile(&profile);
         }
 
         set_common_fields(&container_profile, &self.base);
-        Ok(container_profile)
+
+        container_profile
     }
 
+    #[doc(alias = "gst_encoding_container_profile_add_profile")]
     pub fn add_profile<P: IsA<EncodingProfile>>(mut self, profile: &P) -> Self {
         self.profiles.push(profile.as_ref().clone());
         self
@@ -529,22 +576,22 @@ mod tests {
     fn test_encoding_audio_profile_builder() {
         gst::init().unwrap();
 
-        let caps = gst::Caps::new_simple("audio/x-raw", &[]);
+        let caps = gst::Caps::builder("audio/x-raw").build();
 
-        let restriction = gst::Caps::new_simple("audio/x-raw", &[("format", &"S32LE")]);
+        let restriction = gst::Caps::builder("audio/x-raw")
+            .field("format", "S32LE")
+            .build();
 
-        let audio_profile = EncodingAudioProfileBuilder::new()
+        let audio_profile = EncodingAudioProfile::builder(&caps)
             .name(AUDIO_PROFILE_NAME)
             .description(AUDIO_PROFILE_DESCRIPTION)
-            .format(&caps)
             .preset(PRESET)
             .preset_name(PRESET_NAME)
             .restriction(&restriction)
             .presence(PRESENCE)
             .allow_dynamic_output(ALLOW_DYNAMIC_OUTPUT)
             .enabled(ENABLED)
-            .build()
-            .unwrap();
+            .build();
 
         assert_eq!(audio_profile.name().unwrap(), AUDIO_PROFILE_NAME);
         assert_eq!(
@@ -559,7 +606,9 @@ mod tests {
         assert_eq!(audio_profile.allows_dynamic_output(), ALLOW_DYNAMIC_OUTPUT);
         assert_eq!(audio_profile.is_enabled(), ENABLED);
 
-        let restriction = gst::Caps::new_simple("audio/x-raw", &[("format", &"S32BE")]);
+        let restriction = gst::Caps::builder("audio/x-raw")
+            .field("format", "S32BE")
+            .build();
         audio_profile.set_restriction(Some(&restriction));
         assert_eq!(audio_profile.restriction().unwrap(), restriction);
     }
@@ -568,14 +617,15 @@ mod tests {
     fn test_encoding_video_profile_builder() {
         gst::init().unwrap();
 
-        let caps = gst::Caps::new_simple("video/x-raw", &[]);
+        let caps = gst::Caps::builder("video/x-raw").build();
 
-        let restriction = gst::Caps::new_simple("video/x-raw", &[("format", &"RGBA")]);
+        let restriction = gst::Caps::builder("video/x-raw")
+            .field("format", "RGBA")
+            .build();
 
-        let video_profile = EncodingVideoProfileBuilder::new()
+        let video_profile = EncodingVideoProfile::builder(&caps)
             .name(VIDEO_PROFILE_NAME)
             .description(VIDEO_PROFILE_DESCRIPTION)
-            .format(&caps)
             .preset(PRESET)
             .preset_name(PRESET_NAME)
             .restriction(&restriction)
@@ -584,8 +634,7 @@ mod tests {
             .enabled(ENABLED)
             .pass(PASS)
             .variable_framerate(VARIABLE_FRAMERATE)
-            .build()
-            .unwrap();
+            .build();
 
         assert_eq!(video_profile.name().unwrap(), VIDEO_PROFILE_NAME);
         assert_eq!(
@@ -605,7 +654,9 @@ mod tests {
         assert_eq!(video_profile.is_variableframerate(), VARIABLE_FRAMERATE);
         assert_eq!(video_profile.pass(), PASS);
 
-        let restriction = gst::Caps::new_simple("video/x-raw", &[("format", &"NV12")]);
+        let restriction = gst::Caps::builder("video/x-raw")
+            .field("format", "NV12")
+            .build();
         video_profile.set_restriction(Some(&restriction));
         assert_eq!(video_profile.restriction().unwrap(), restriction);
     }
@@ -614,27 +665,22 @@ mod tests {
     fn test_encoding_container_profile_builder() {
         gst::init().unwrap();
 
-        let container_caps = gst::Caps::new_simple("container/x-caps", &[]);
-        let video_caps = gst::Caps::new_simple("video/x-raw", &[]);
-        let audio_caps = gst::Caps::new_simple("audio/x-raw", &[]);
+        let container_caps = gst::Caps::builder("container/x-caps").build();
+        let video_caps = gst::Caps::builder("video/x-raw").build();
+        let audio_caps = gst::Caps::builder("audio/x-raw").build();
 
-        let video_profile = EncodingVideoProfileBuilder::new()
+        let video_profile = EncodingVideoProfile::builder(&video_caps)
             .name(VIDEO_PROFILE_NAME)
             .description(VIDEO_PROFILE_DESCRIPTION)
-            .format(&video_caps)
-            .build()
-            .unwrap();
-        let audio_profile = EncodingAudioProfileBuilder::new()
+            .build();
+        let audio_profile = EncodingAudioProfile::builder(&audio_caps)
             .name(AUDIO_PROFILE_NAME)
             .description(AUDIO_PROFILE_DESCRIPTION)
-            .format(&audio_caps)
-            .build()
-            .unwrap();
+            .build();
 
-        let profile = EncodingContainerProfileBuilder::new()
+        let profile = EncodingContainerProfile::builder(&container_caps)
             .name(CONTAINER_PROFILE_NAME)
             .description(CONTAINER_PROFILE_DESCRIPTION)
-            .format(&container_caps)
             .preset(PRESET)
             .preset_name(PRESET_NAME)
             .presence(PRESENCE)
@@ -642,8 +688,7 @@ mod tests {
             .enabled(ENABLED)
             .add_profile(&audio_profile)
             .add_profile(&video_profile)
-            .build()
-            .unwrap();
+            .build();
 
         assert_eq!(profile.name().unwrap(), CONTAINER_PROFILE_NAME);
         assert_eq!(
